from operator import gt
from unittest import getTestCaseNames
import pandas as pd
import numpy as np
import requests
import io
import os
import sys
import tqdm
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
from multiprocessing import Pool
from functools import partial
import re
import time
import tarfile
import shutil
import pathlib
from get_vf.defaults import VFDB, vfdbs
import logging
from datetime import date
import gzip
from Bio import SeqIO
import subprocess


def create_mmseqs_db(db_dir, db, faa, mmseqs_bin):
    # Create mmseqs2 db
    mmseqs_dir = pathlib.Path(db_dir, "mmseqs")
    mmseqs_db = pathlib.Path(mmseqs_dir, f"{db}-db")
    if not os.path.exists(mmseqs_db):
        create_folder(mmseqs_dir)
        create_mmseqs_db_worker(
            faa=faa,
            mmseqs_bin=mmseqs_bin,
            mmseqs_db=mmseqs_db,
            db=db,
        )
    else:
        logging.info("MMseqs2 DB already built. Skipping creating DB.")


def create_mmseqs_db_worker(faa, mmseqs_bin, mmseqs_db, db):
    logging.info(f"Creating aa MMseqs2 DB for {db} VFDB")
    proc = subprocess.Popen(
        [
            mmseqs_bin,
            "createdb",
            faa,
            mmseqs_db,
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    stdout, stderr = proc.communicate()
    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error creating mmseqs DB for {db}")
        logging.error(stderr)
        exit(1)


def parse_header(header):
    # Use a regular expression to extract the gene name, description, virulence factor, and organism
    match = re.match(r"(\S+)\s(\S.+)\[(\S.+)\]\s\[(\S.+)\]", header)

    # If the regular expression matched, return the extracted information
    if match:
        return (match.group(1), match.group(2), match.group(3), match.group(4))
    # If the regular expression did not match, return None
    else:
        return None


# def create_output_files(outdir):
#     # create output files
#     out_files = {
#         "core": {
#             "out_tsv": f"{outdir}/core/VFDB.metadata.tsv.gz",
#             "out_fasta": f"{outdir}/core/VFDB.fasta.gz",
#         },
#         "full": {
#             "out_tsv": f"{outdir}/full/VFDB.metadata.tsv.gz",
#             "out_fasta": f"{outdir}/full/VFDB.full.fasta.gz",
#         },
#     }
#     return out_files


def create_db_files(input, faa, metadata):
    with open(input, "rb") as handle:
        # Read the first two bytes of the file
        magic = handle.read(2)

        # If the first two bytes are the GZIP magic number, the file is compressed
        if magic == b"\x1f\x8b":
            mode = "rt"
        else:
            mode = "r"

        # Seek back to the beginning of the file
        handle.seek(0)

        # Open the file with gzip.open, using the determined mode
        with gzip.open(handle, mode) as handle:
            # Open the output file
            with gzip.open(metadata, "wt") as output_tsv, gzip.open(
                faa, "wt"
            ) as fasta_output:
                # Loop through the sequences in the FASTA file
                for record in SeqIO.parse(handle, "fasta"):
                    # Parse the header and extract the information
                    info = parse_header(record.description)
                    # If the header was successfully parsed, write the extracted information to the output file
                    if info:
                        string = f"{info[0]}\t{info[2]}\n"
                        record.id = info[0]
                        record.description = ""
                        output_tsv.write(string)
                        SeqIO.write(record, handle=fasta_output, format="fasta")


def get_data(url):
    # Get the data from the API
    retry_strategy = Retry(
        total=10,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504, 403],
        allowed_methods=["HEAD", "GET", "OPTIONS"],
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    http = requests.Session()
    http.mount("https://", adapter)
    http.mount("http://", adapter)
    r = http.get(url, timeout=10)
    # Parse the data
    if r.status_code != 200:
        r = io.StringIO(f"Error: {r.status_code}")
    else:
        r = io.StringIO(r.content.decode("utf-8"))
    return r


def create_folder(path):
    if not os.path.exists(path):
        os.makedirs(path)
    else:
        log.info(f"Folder {path} already exists. Skipping...")


def create_folder_and_delete(path):
    if not os.path.exists(path):
        os.makedirs(path)
    else:
        log.info(f"Folder {path} already exists. Deleting...")
        try:
            shutil.rmtree(path)
            os.makedirs(path)
        except OSError as e:
            log.error(f"{path} : {e.strerror}")


def is_folder_empty(path):
    with os.scandir(path) as it:
        if any(it):
            return False
        else:
            return True


def delete_folder(path):
    if os.path.exists(path):
        log.info(f"Deleting folder {path}...")
        try:
            shutil.rmtree(path)
        except OSError as e:
            log.error(f"{path} : {e.strerror}")


# Function to connect to the KEGG API and return the genomes in TSV format
def download(url: str, dest_folder: str):
    if not os.path.exists(dest_folder):
        os.makedirs(dest_folder)  # create folder if it does not exist

    filename = os.path.basename(url)
    file_path = os.path.join(dest_folder, filename)
    chunk_size = 1024
    filesize = int(requests.head(url).headers["Content-Length"])
    with requests.get(url, stream=True) as r, open(file_path, "wb") as f, tqdm.tqdm(
        unit="B",  # unit string to be displayed.
        unit_scale=True,  # let tqdm to determine the scale in kilo, mega..etc.
        unit_divisor=1024,  # is used when unit_scale is true
        total=filesize,  # the total iteration.
        file=sys.stdout,  # default goes to stderr, this is the display on console.
        desc=filename,  # prefix to be displayed on progress bar.
    ) as progress:
        for chunk in r.iter_content(chunk_size=chunk_size):
            # download the file chunk by chunk
            datasize = f.write(chunk)
            # on each chunk update the progress bar.
            progress.update(datasize)


log = logging.getLogger("my_logger")


def createdb(args):
    # check if output directory exists if not create it else delete it
    outdir = f"{args.createdb_output}/DB"
    core_dir = f"{outdir}/core"
    full_dir = f"{outdir}/full"
    # out_files = create_output_files(outdir)

    # check if output directory exists if not create it
    create_folder(outdir)
    # check if the subfolders are in place, if they exist remove
    # create_folder_and_delete(faa_dir)
    # create_folder_and_delete(fna_dir)
    # create_folder_and_delete(hmm_dir)
    if args.recreate:
        delete_folder(core_dir)
        delete_folder(full_dir)
        create_folder(core_dir)
        create_folder(full_dir)
    else:
        create_folder(core_dir)
        create_folder(full_dir)

    # create a temporary directory to store the downloaded files
    if args.createdb_tmp is None:
        tmp_dir = f"{args.createdb_output}/tmp"
    else:
        tmp_dir = args.createdb_tmp
    create_folder(tmp_dir)

    # Get the date of the latest release
    today_date = date.today()
    version_file = pathlib.Path(f"{outdir}/version")
    with open(version_file, "w") as f:
        f.write(f"{today_date}")
    # Download the files to the tmp directory
    for db in vfdbs:
        log.info(f"Downloading VFDB {db} database")
        download(VFDB[db]["url"], tmp_dir)

    # Create metadata file
    log.info("Processing files")
    for db in vfdbs:
        create_db_files(
            input=f"{tmp_dir}/{VFDB[db]['filename']}",
            faa=pathlib.Path(outdir, db, VFDB[db]["faa"]),
            metadata=pathlib.Path(outdir, db, VFDB[db]["metadata"]),
        )
        create_mmseqs_db(
            db_dir=core_dir,
            db=db,
            faa=pathlib.Path(outdir, db, VFDB[db]["faa"]),
            mmseqs_bin=args.mmseqs_bin,
        )

    # Delete the temporary directory
    log.info("Removing temporary directory")
    delete_folder(tmp_dir)

    log.info("Done")



import sys
import logging
import pathlib
import pandas as pd
import os
import requests
import http.client
from Bio import SeqIO, AlignIO, Seq
import numpy as np
from scipy import stats
import subprocess
import re
import io
from multiprocessing import Pool
from get_vf.utils import (
    create_folder,
    create_folder_and_delete,
    is_fastq,
    create_output_files,
    get_date_dir,
    delete_folder,
)
from get_vf.defaults import VFDB, vfdbs
import gzip
import shutil
from contextlib import ExitStack

log = logging.getLogger("my_logger")


def extend_reads(
    reads,
    output,
    extend_bin,
    extend_k,
    extend_length,
    extend_memory,
    tmp_dir,
    threads,
    db,
    log_file,
):
    if extend_memory is None:
        extend_memory = " "
    else:
        extend_memory = f"-Xmx{extend_memory}"

    output = pathlib.Path(tmp_dir, output)
    if output.exists():
        logging.info("Extending reads already found. Skipping.")
        return output
    logging.info(f"Extending reads [k:{extend_k}; length:{extend_length}]")
    proc = subprocess.Popen(
        [
            extend_bin,
            extend_memory,
            f"in={reads}",
            f"out={output}",
            f"k={extend_k}",
            f"mode=extend",
            f"el={extend_length}",
            f"er={extend_length}",
            f"overwrite=true",
            f"threads={threads}",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    stdout, stderr = proc.communicate()

    with open(log_file, "w") as f:
        for row in stderr.split("\n"):
            f.write(row + "\n")

    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error extending reads for DB: {db}")
        logging.error(stderr)
        exit(1)
    return output


def dereplicate_reads(
    derep_bin,
    reads,
    output,
    tmp_dir,
    log_file,
    db,
    # derep_min_length,
    threads,
):
    # vsearch --derep_fulllength ${i} --output - --minseqlength 30 --strand both
    output = pathlib.Path(tmp_dir, output)
    if output.exists():
        logging.info("Dereplicated reads already found. Skipping.")
        return output
    if derep_bin == "vsearch":
        logging.info(f"Dereplicating reads with {derep_bin}")
        proc = subprocess.Popen(
            [
                derep_bin,
                "--fastx_uniques",
                reads,
                "--fastqout",
                output,
                "--strand",
                "both",
                # "--minseqlength",
                # str(derep_min_length),
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding="utf-8",
        )
        stdout, stderr = proc.communicate()

    elif derep_bin == "seqkit":
        logging.info(f"Dereplicating reads with {derep_bin}")
        proc = subprocess.Popen(
            [
                derep_bin,
                "rmdup",
                "-s",
                reads,
                "-j",
                str(threads),
                "-o",
                output,
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding="utf-8",
        )
        stdout, stderr = proc.communicate()

    with open(log_file, "w") as f:
        for row in stderr.split("\n"):
            f.write(row + "\n")
    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error dereplicating reads for DB: {db}")
        logging.error(stderr)
        exit(1)
    return output


def search_mmseqs(
    reads,
    db,
    vfdb_db,
    output,
    tmp_dir,
    log_file,
    mmseqs_bin,
    mmseqs_min_length,
    mmseqs_evalue,
    mmseqs_min_seqid,
    mmseqs_cov,
    mmseqs_cov_mode,
    ancient,
    threads,
):
    tmp = pathlib.Path(tmp_dir, "mmseqs", db)
    create_folder(tmp)
    if output.exists():
        output.unlink()
    logging.info(f"Searching reads against DB {db}")
    if ancient:
        logging.info("::: Using MMseqs2 parameters for aDNA")
        cmd = [
            mmseqs_bin,
            "easy-search",
            reads,
            vfdb_db,
            output,
            tmp,
            "--min-length",
            str(mmseqs_min_length),
            "-e",
            str(mmseqs_evalue),
            "--min-seq-id",
            str(mmseqs_min_seqid),
            "-c",
            str(mmseqs_cov),
            "--cov-mode",
            str(mmseqs_cov_mode),
            "--format-mode",
            str(2),
            "--comp-bias-corr",
            str(0),
            "--mask",
            str(0),
            "--exact-kmer-matching",
            str(1),
            "--sub-mat",
            "PAM30.out",
            "-s",
            str(5),
            "-k",
            str(6),
            "--spaced-kmer-pattern",
            "11011101",
            "--threads",
            str(threads),
        ]
    else:
        cmd = [
            mmseqs_bin,
            "easy-search",
            reads,
            vfdb_db,
            output,
            tmp,
            "--min-length",
            str(mmseqs_min_length),
            "-e",
            str(mmseqs_evalue),
            "--min-seq-id",
            str(mmseqs_min_seqid),
            "-c",
            str(mmseqs_cov),
            "--cov-mode",
            str(mmseqs_cov_mode),
            "--format-mode",
            str(2),
            "--threads",
            str(threads),
        ]
    proc = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    stdout, stderr = proc.communicate()

    with open(log_file, "w") as f:
        for row in stderr.split("\n"):
            f.write(row + "\n")

    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error searching reads for DB: {db}")
        logging.error(stderr)
        exit(1)
    delete_folder(tmp)
    return output


def filter_results(
    xfilter_bin,
    results,
    prefix,
    db,
    log_file,
    n_iters,
    evalue,
    scale,
    bitscore,
    fltr,
    breadth,
    breadth_expected_ratio,
    depth,
    depth_evenness,
    trim,
    threads,
    output,
    metadata,
    aggregate,
):
    logging.info(f"Filtering VFDB {db} BLASTx results")
    if trim:
        if aggregate:
            cmd = [
                xfilter_bin,
                "--input",
                results,
                "--prefix",
                prefix,
                "--n-iters",
                str(n_iters),
                "--evalue",
                str(evalue),
                "--scale",
                str(scale),
                "--bitscore",
                str(bitscore),
                "--filter",
                str(fltr),
                "--breadth",
                str(breadth),
                "--breadth-expected-ratio",
                str(breadth_expected_ratio),
                "--depth",
                str(depth),
                "--depth-evenness",
                str(depth_evenness),
                "--threads",
                str(threads),
                "-m",
                metadata,
            ]
        else:
            cmd = [
                xfilter_bin,
                "--input",
                results,
                "--prefix",
                prefix,
                "--n-iters",
                str(n_iters),
                "--evalue",
                str(evalue),
                "--scale",
                str(scale),
                "--bitscore",
                str(bitscore),
                "--filter",
                str(fltr),
                "--breadth",
                str(breadth),
                "--breadth-expected-ratio",
                str(breadth_expected_ratio),
                "--depth",
                str(depth),
                "--depth-evenness",
                str(depth_evenness),
                "--threads",
                str(threads),
            ]
    else:
        if aggregate:
            cmd = [
                xfilter_bin,
                "--input",
                results,
                "--prefix",
                prefix,
                "--n-iters",
                str(n_iters),
                "--evalue",
                str(evalue),
                "--scale",
                str(scale),
                "--bitscore",
                str(bitscore),
                "--filter",
                str(fltr),
                "--breadth",
                str(breadth),
                "--breadth-expected-ratio",
                str(breadth_expected_ratio),
                "--depth",
                str(depth),
                "--depth-evenness",
                str(depth_evenness),
                "--threads",
                str(threads),
                "--no-trim",
                "-m",
                metadata,
            ]
        else:
            cmd = [
                xfilter_bin,
                "--input",
                results,
                "--prefix",
                prefix,
                "--n-iters",
                str(n_iters),
                "--evalue",
                str(evalue),
                "--scale",
                str(scale),
                "--bitscore",
                str(bitscore),
                "--filter",
                str(fltr),
                "--breadth",
                str(breadth),
                "--breadth-expected-ratio",
                str(breadth_expected_ratio),
                "--depth",
                str(depth),
                "--depth-evenness",
                str(depth_evenness),
                "--threads",
                str(threads),
                "--no-trim",
            ]
    proc = subprocess.Popen(
        cmd,
        cwd=output,
        stdout=subprocess.PIPE,
        # stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    stdout, stderr = proc.communicate()

    # with open(log_file, "w") as f:
    #     for row in stderr.split("\n"):
    #         f.write(row + "\n")

    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error filtering BLASTx results for DB: {db}")
        # logging.error(stderr)
        exit(1)
    # gzip output file if exists


def extract_reads(results, db, output, threads, reads, extract_bin, tmp_dir, log_file):
    logging.info(f"Extracting reads for DB {db}")
    results = pd.read_csv(results, sep="\t")
    name_lst = pathlib.Path(tmp_dir, f"{db}-names.tsv")
    results[["queryId"]].to_csv(name_lst, sep="\t", index=False, header=False)
    cmd = [
        extract_bin,
        f"in={reads}",
        f"out={output}",
        f"names={name_lst}",
        f"overwrite=true",
        f"threads={threads}",
        f"include=t",
    ]

    proc = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )

    stdout, stderr = proc.communicate()

    with open(log_file, "w") as f:
        for row in stderr.split("\n"):
            f.write(row + "\n")

    if proc.returncode != 0:
        # rgx = re.compile("ERROR")
        # for line in stdout.splitlines():
        #     if rgx.search(line):
        #         error = line.replace("ERROR: ", "")
        logging.error(f"Error extracting reads for DB: {db}")
        logging.error(stderr)
        exit(1)
    return output


def search_db(args):

    if args.no_derep:
        derep = False
    else:
        derep = True

    if args.no_extend:
        extend = False
    else:
        extend = True

    if args.no_filter:
        fltr = False
    else:
        fltr = True

    vfdb_db = pathlib.Path(
        args.db_dir,
        args.db,
        "mmseqs",
        f"{args.db}-db",
    )

    metadata = pathlib.Path(args.db_dir, args.db, VFDB[args.db]["metadata"]).resolve()
    if not os.path.exists(vfdb_db):
        logging.error(
            f"The DB for the {args.db} VFDB does not exist. Please run the createdb subcommand to create it."
        )
        exit(1)
    else:
        logging.info(f"DB for the {args.db} VFDB found")

    logging.info(f"Creating output and tmp folders")
    date_dir = get_date_dir()

    output = pathlib.Path(args.output, args.db)
    create_folder(output)
    if args.tmp is None:
        tmp_dir = pathlib.Path(output, "tmp")
        create_folder(tmp_dir)
    else:
        tmp_dir = pathlib.Path(args.tmp, args.db, date_dir, "tmp")
        create_folder(tmp_dir)

    fastq = is_fastq(args.input)
    output_files = create_output_files(
        prefix=args.prefix, input=args.input, fastq=fastq, db=args.db
    )

    log_dir = pathlib.Path(output, "logs")
    create_folder(log_dir)
    extend_log_file = pathlib.Path(log_dir, output_files["extend_reads_log"])

    extended_reads = pathlib.Path(tmp_dir, output_files["extended_reads"])
    if extend:
        if not os.path.exists(extended_reads):
            # extend
            extend_reads(
                reads=args.input,
                output=output_files["extended_reads"],
                extend_bin=args.extend_bin,
                extend_k=args.extend_k,
                extend_length=args.extend_length,
                extend_memory=args.extend_memory,
                threads=args.threads,
                tmp_dir=tmp_dir,
                db=args.db,
                log_file=extend_log_file,
            )
        else:
            logging.info("Extended reads already exists. Skipping.")
    # dereplicate
    derep_log_file = pathlib.Path(log_dir, output_files["derep_reads_log"])
    derep_reads = pathlib.Path(tmp_dir, output_files["derep_reads"])

    if extend and os.path.exists(extended_reads) and derep:
        dereplicate_reads(
            reads=extended_reads,
            db=args.db,
            output=output_files["derep_reads"],
            derep_bin=args.derep_bin,
            tmp_dir=tmp_dir,
            log_file=derep_log_file,
            # derep_min_length=args.derep_min_length,
            threads=args.threads,
        )
    elif derep:
        dereplicate_reads(
            reads=args.input,
            db=args.db,
            output=output_files["derep_reads"],
            derep_bin=args.derep_bin,
            tmp_dir=tmp_dir,
            log_file=derep_log_file,
            # derep_min_length=args.derep_min_length,
            threads=args.threads,
        )
    # search
    if os.path.exists(derep_reads):
        reads = derep_reads
    elif os.path.exists(extended_reads) and not os.path.exists(derep_reads):
        reads = extended_reads
    else:
        reads = args.input

    results = pathlib.Path(output, output_files["results"])
    results_gz = pathlib.Path(output, output_files["results_gz"])
    search_log_file = pathlib.Path(log_dir, output_files["results_log"])
    if not os.path.exists(results_gz):
        search_mmseqs(
            reads=reads,
            db=args.db,
            vfdb_db=vfdb_db,
            output=results,
            tmp_dir=tmp_dir,
            log_file=search_log_file,
            mmseqs_bin=args.mmseqs_bin,
            mmseqs_min_length=args.mmseqs_min_length,
            mmseqs_evalue=args.mmseqs_evalue,
            mmseqs_min_seqid=args.mmseqs_min_seqid,
            mmseqs_cov=args.mmseqs_cov,
            mmseqs_cov_mode=args.mmseqs_cov_mode,
            ancient=args.ancient,
            threads=args.threads,
        )
    else:
        logging.info("Results already exists. Skipping.")

    if os.path.exists(results):
        with ExitStack() as stack:
            f_in = stack.enter_context(open(results, "rb"))
            f_out = stack.enter_context(gzip.open(results_gz, "wb"))
            shutil.copyfileobj(f_in, f_out)
        results.unlink()
    # filter
    extract_log_file = pathlib.Path(log_dir, output_files["reads_db_log"])
    reads_db = pathlib.Path(output, output_files["reads_db"])
    if fltr and os.path.exists(results_gz):
        filter_log_file = pathlib.Path(log_dir, output_files["results_filtered_log"])
        filter_results(
            results=output_files["results_gz"],
            log_file=filter_log_file,
            n_iters=args.iters,
            evalue=args.evalue,
            scale=args.scale,
            bitscore=args.bitscore,
            fltr=args.filter,
            breadth=args.breadth,
            breadth_expected_ratio=args.breadth_expected_ratio,
            depth=args.depth,
            depth_evenness=args.depth_evenness,
            trim=args.trim,
            db=args.db,
            threads=args.threads,
            xfilter_bin=args.xfilter_bin,
            prefix=output_files["results_filtered_prefix"],
            output=output,
            metadata=metadata,
            aggregate=args.agg,
        )
        cov_file = pathlib.Path(output, output_files["results_filtered_cov"]).resolve()
        mm_file = pathlib.Path(output, output_files["results_filtered_mm"]).resolve()
        # group_file = pathlib.Path(output, args.db, output_files["results_filtered_group"])
        # group_file_agg = pathlib.Path(output, args.db, output_files["results_filtered_group_agg"])

        if not os.path.exists(cov_file) or not os.path.exists(mm_file):
            logging.error("Cannot find filtered results. Exiting.")
            exit(1)
        if args.extract:
            if os.path.exists(mm_file):
                extract_reads(
                    results=mm_file,
                    db=args.db,
                    output=reads_db,
                    threads=args.threads,
                    reads=args.input,
                    extract_bin=args.extract_bin,
                    tmp_dir=tmp_dir,
                    log_file=extract_log_file,
                )
            else:
                if os.path.exists(results_gz):
                    extract_reads(
                        results=results_gz,
                        db=args.db,
                        output=reads_db,
                        threads=args.threads,
                        reads=args.input,
                        extract_bin=args.extract_bin,
                        tmp_dir=tmp_dir,
                        log_file=extract_log_file,
                    )

    # Clean up temporary
    if args.no_keep and os.path.exists(tmp_dir):
        logging.info("Removing temporary files.")
        shutil.rmtree(tmp_dir)



VFDB = {
    "core": {
        "url": "http://www.mgc.ac.cn/VFs/Down/VFDB_setA_pro.fas.gz",
        "filename": "VFDB_setA_pro.fas.gz",
        "metadata": "VFDB.core.metadata.tsv.gz",
        "faa": "VFDB.core.fasta.gz",
    },
    "full": {
        "url": "http://www.mgc.ac.cn/VFs/Down/VFDB_setB_pro.fas.gz",
        "filename": "VFDB_setB_pro.fas.gz",
        "metadata": "VFDB.full.metadata.tsv.gz",
        "faa": "VFDB.full.fasta.gz",
    },
}


vfdbs = ["core", "full"]

derep_bins = ["vsearch", "seqkit"]



import argparse
import sys
import gzip
import os
import shutil
import logging
import pandas as pd
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from os import devnull
from get_vf import __version__
from itertools import chain
from pathlib import Path
from get_vf.defaults import vfdbs, derep_bins

# from get_vf.defaults import markers, aln_formats, gtdb_releases
from Bio import SeqIO
import datetime


def is_debug():
    return logging.getLogger("my_logger").getEffectiveLevel() == logging.DEBUG


filters = ["breadth", "depth", "depth_evenness", "breadth_expected_ratio"]

# From https://stackoverflow.com/a/59617044/15704171
def convert_list_to_str(lst):
    n = len(lst)
    if not n:
        return ""
    if n == 1:
        return lst[0]
    return ", ".join(lst[:-1]) + f" or {lst[-1]}"


def check_filter_values(val, parser, var):
    value = str(val)
    if value in filters:
        return value
    else:
        parser.error(
            f"argument {var}: Invalid value {value}. Filter has to be one of {convert_list_to_str(filters)}"
        )


def check_values(val, minval, maxval, parser, var):
    value = float(val)
    if value < minval or value > maxval:
        parser.error(
            f"argument {var}: Invalid value value. Range has to be between {minval} and {maxval}!"
        )
    return value


def get_date_dir():
    current_date = datetime.datetime.now()
    folder_name = current_date.strftime("%Y%m%d_%H-%M-%S")
    return folder_name


# From: https://note.nkmk.me/en/python-check-int-float/
def is_integer(n):
    try:
        float(n)
    except ValueError:
        return False
    else:
        return float(n).is_integer()


# function to check if the input value has K, M or G suffix in it
def check_suffix(val, parser, var):
    units = ["K", "M", "G"]
    unit = val[-1]
    value = int(val[:-1])

    if is_integer(value) & (unit in units) & (value > 0):
        return val
    else:
        parser.error(
            "argument %s: Invalid value %s. Memory has to be an integer larger than 0 with the following suffix K, M or G"
            % (var, val)
        )


def get_compression_type(filename):
    """
    Attempts to guess the compression (if any) on a file using the first few bytes.
    http://stackoverflow.com/questions/13044562
    """
    magic_dict = {
        "gz": (b"\x1f", b"\x8b", b"\x08"),
        "bz2": (b"\x42", b"\x5a", b"\x68"),
        "zip": (b"\x50", b"\x4b", b"\x03", b"\x04"),
    }
    max_len = max(len(x) for x in magic_dict)

    unknown_file = open(filename, "rb")
    file_start = unknown_file.read(max_len)
    unknown_file.close()
    compression_type = "plain"
    for file_type, magic_bytes in magic_dict.items():
        if file_start.startswith(magic_bytes):
            compression_type = file_type
    if compression_type == "bz2":
        sys.exit("Error: cannot use bzip2 format - use gzip instead")
        sys.exit("Error: cannot use zip format - use gzip instead")
    return compression_type


def get_open_func(filename):
    if get_compression_type(filename) == "gz":
        return gzip.open
    else:  # plain text
        return open


# From: https://stackoverflow.com/a/11541450
def is_valid_file(parser, arg, var):
    if not os.path.exists(arg):
        parser.error("argument %s: The file %s does not exist!" % (var, arg))
    else:
        return arg


def is_valid_dir(parser, arg, var):
    if not os.path.exists(arg):
        parser.error("argument %s: The folder %s does not exist!" % (var, arg))
    else:
        return arg


def is_executable(parser, arg, var):
    """Check whether `name` is on PATH and marked as executable."""
    if shutil.which(arg):
        return arg
    else:
        parser.error("argument %s: %s not found in path." % (var, arg))


def is_in_vfdb(parser, arg, var):
    if arg in vfdbs:
        return arg
    else:
        parser.error(
            "argument %s: Invalid value %s.\nDB has to be one of %s"
            % (var, arg, convert_list_to_str(vfdbs))
        )


def is_in_derep_bins(parser, arg, var):
    derep_exec = Path(arg).name
    if derep_exec in derep_bins:
        is_executable(parser, arg, var)
        return arg
    else:
        parser.error(
            "argument %s: Invalid value %s.\De-replication binary has to be one of %s"
            % (var, arg, convert_list_to_str(derep_bins))
        )


def create_folder_and_delete(path):
    if not os.path.exists(path):
        os.makedirs(path)
    else:
        logging.info(f"Folder {path} already exists. Deleting.")
        try:
            shutil.rmtree(path)
            os.makedirs(path)
        except OSError as e:
            print("Error: %s : %s" % (path, e.strerror))


def delete_folder(path):
    if os.path.exists(path):
        logging.info(f"Deleting folder {path}.")
        try:
            shutil.rmtree(path)
        except OSError as e:
            print("Error: %s : %s" % (path, e.strerror))
    else:
        logging.info(f"Path {path} does not exist.")


def create_folder(path):
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except OSError as e:
            print("Error: %s : %s" % (path, e.strerror))
    else:
        logging.info(f"Folder {path} already exists. Skipping creation.")


defaults = {
    "db": "core",
    "threads": 1,
    "outdir": "get-ancient-vf-output-fetch",
    "mmseqs_bin": "mmseqs",
    "extend_bin": "tadpole.sh",
    "extend_length": 100,
    "extend_memory": None,
    "extend_k": 17,
    "tmp": None,
    "derep_bin": "seqkit",
    "derep_min_length": 30,
    "createdb_release": "latest",
    "output": "get-ancient-vf-output-search",
    "db_dir": "get-ancient-vf-output-fetch",
    "mmseqs2_bin": "mmseqs",
    "mmseqs2_min_length": 15,
    "mmseqs2_min_seqid": 0.6,
    "mmseqs2_cov": 0.6,
    "mmseqs2_cov_mode": 2,
    "mmseqs2_evalue": 1e-5,
    "bitscore": 60,
    "evalue": 1e-5,
    "breadth": 0.5,
    "breadth_expected_ratio": 0.5,
    "depth": 0.1,
    "depth_evenness": 1.0,
    "prefix": None,
    "sort_memory": "1G",
    "mapping_file": None,
    "iters": 25,
    "scale": 0.9,
    "filter": "depth_evenness",
    "xfilter_bin": "xFilter",
    "extract_bin": "filterbyname.sh",
    "vfdb": "core",
}

help_msg = {
    "input": "A FASTA file containing the query sequences",
    "threads": "Number of threads to use",
    "outdir": "Output folder",
    "local_db": "Path to a local getVF DB",
    "vfdb": "Which VFDB to use",
    "hmmalign_bin": "Path to the hmmalign executable",
    "mmseqs_bin": "Path to the mmseqs2 executable",
    "no_extend": "Disable read extension",
    "extend_bin": "Path to the the executable for the extension step",
    "extend_length": "How much to extend in both ends",
    "extend_memory": "How much memory to use for the extension",
    "extend_k": "K-mer length to use for the extension",
    "createdb_release": "Release where to get the data",
    "createdb_output": "Output folder where to store the DB",
    "recreate": "Remove folders if they exist",
    "output": "Output folder to write the search results",
    "prefix": "Prefix used for the output files",
    "tmp": "Path to the temporary directory",
    "db_dir": "Folder with the VFDB data generated by the createdb subcommand",
    "no_derep": "Disable de-replication of identical reads",
    "derep_bin": "Path to the the executable for the de-replication step",
    "derep_min_length": "Minimum read length for the de-replication step",
    "mmseqs2_bin": "mmseqs",
    "ancient": "Use mmseqs2 aDNA optimized parameters",
    "mmseqs2_min_length": "Minimum length of the predicted ORF",
    "mmseqs2_min_seqid": "List matches above this sequence identity",
    "mmseqs2_cov": "List matches above this fraction of aligned (covered) residues",
    "mmseqs2_cov_mode": "Type of mmseq2 coverage calculation",
    "mmseqs2_evalue": " List matches below this E-value",
    "no_filter": "Disable x-filter filtering",
    "bitscore": "Bitscore where to filter the results",
    "evalue": "Evalue where to filter the results",
    "filter": "Which filter to use. Possible values are: breadth, depth, depth_evenness, breadth_expected_ratio",
    "xfilter_bin": "Path to the the executable for the x-filter step",
    "breadth": "Breadth of the coverage",
    "breadth_expected_ratio": "Expected breath to observed breadth ratio (scaled)",
    "depth": "Depth to filter out",
    "depth_evenness": "Reference with higher evenness will be removed",
    "mapping_file": "File with mappings to genes for aggregation",
    "iters": "Number of iterations for the FAMLI-like filtering",
    "scale": "Scale to select the best weithing alignments",
    "agg": "Aggregate the results by virulence factor",
    "help": "Help message",
    "debug": f"Print debug messages",
    "version": f"Print program version",
    "trim": f"Deactivate the trimming for the coverage calculations",
    "extract": f"Extract the reads that match the VFDB",
    "extract_bin": "Path to the the executable for the read extraction step",
    "keep": "Keep temporary files",
}


def get_arguments(argv=None):
    parser = argparse.ArgumentParser(
        description="A simple tool to filter BLASTx m8 files using the FAMLI algorithm",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s " + __version__,
        help=help_msg["version"],
    )
    parser.add_argument(
        "--debug", dest="debug", action="store_true", help=help_msg["debug"]
    )

    # Same subparsers as usual
    sub_parsers = parser.add_subparsers(
        help="positional arguments",
        dest="action",
    )

    # Create parent subparser. Note `add_help=False` and creation via `argparse.`
    parent_parser = argparse.ArgumentParser(add_help=False)
    optional = parent_parser._action_groups.pop()

    optional.add_argument(
        "--threads",
        type=lambda x: int(
            check_values(x, minval=1, maxval=1000, parser=parser, var="--threads")
        ),
        dest="threads",
        default=1,
        help=help_msg["threads"],
    )
    # Create the parser sub-command for db creation
    parser_createdb = sub_parsers.add_parser(
        "createdb",
        help="Gather data from GTDB and create a DB",
        parents=[parent_parser],
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    # create the parser sub-command for search
    parser_search = sub_parsers.add_parser(
        "search",
        help="Search short reads against VFDB",
        parents=[parent_parser],
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    # createdb_required_args = parser_createdb.add_argument_group("required arguments")
    createdb_required_args = parser_createdb.add_argument_group(
        "Createdb required arguments"
    )
    createdb_optional_args = parser_createdb.add_argument_group(
        "Createdb optional arguments"
    )

    search_required_args = parser_search.add_argument_group("Search required arguments")
    search_optional_args = parser_search.add_argument_group("Search optional arguments")
    extend_args = parser_search.add_argument_group("Read extension arguments")
    derep_args = parser_search.add_argument_group("Read dereplication arguments")
    search_args = parser_search.add_argument_group("MMseqs2 search arguments")
    filter_args = parser_search.add_argument_group("X-filter processing arguments")
    extract_args = parser_search.add_argument_group(
        "Extract VFDB mapping reads arguments"
    )

    # createdb_required_args.add_argument(
    #     "-r",
    #     "--release",
    #     metavar="STR",
    #     type=lambda x: is_valid_gtdb_release(x, parser, "--release"),
    #     required=True,
    #     help=help_msg["createdb_release"],
    #     dest="gtdb_release",
    #     default=defaults["createdb_release"],
    # )
    createdb_output = get_date_dir()
    createdb_optional_args.add_argument(
        "--output",
        type=str,
        required=True,
        # default=createdb_output,
        metavar="STR",
        dest="createdb_output",
        help=help_msg["createdb_output"],
    )
    createdb_optional_args.add_argument(
        "--tmp",
        type=str,
        required=False,
        metavar="STR",
        default=None,
        help="Temporary directory location",
        dest="createdb_tmp",
    )
    createdb_optional_args.add_argument(
        "--recreate",
        dest="recreate",
        action="store_true",
        help=help_msg["recreate"],
        default=argparse.SUPPRESS,
    )
    createdb_optional_args.add_argument(
        "--mmseqs-bin",
        default=defaults["mmseqs_bin"],
        metavar="STR",
        type=lambda x: is_executable(parser, x, "--mmseqs-bin"),
        dest="mmseqs_bin",
        help=help_msg["mmseqs_bin"],
    )
    search_required_args.add_argument(
        "--input",
        required=True,
        default=argparse.SUPPRESS,
        type=lambda x: is_valid_file(parser, x, "--input"),
        help=help_msg["input"],
    )
    search_required_args.add_argument(
        "--vfdb",
        required=True,
        metavar="STR",
        default=defaults["vfdb"],
        type=lambda x: is_in_vfdb(parser, x, "--vfdb"),
        dest="db",
        help=help_msg["vfdb"],
    )
    search_required_args.add_argument(
        "--db-dir",
        required=True,
        metavar="STR",
        default=argparse.SUPPRESS,
        type=lambda x: is_valid_file(parser, x, "--db-dir"),
        dest="db_dir",
        help=help_msg["db_dir"],
    )
    search_optional_args.add_argument(
        "--output",
        type=str,
        default=defaults["output"],
        metavar="STR",
        dest="output",
        help=help_msg["output"],
    )
    search_optional_args.add_argument(
        "--tmp",
        type=lambda x: is_valid_file(parser, x, "--tmp"),
        default=defaults["tmp"],
        metavar="STR",
        dest="tmp",
        help=help_msg["tmp"],
    )
    search_optional_args.add_argument(
        "--prefix",
        type=str,
        metavar="STR",
        default=defaults["prefix"],
        dest="prefix",
        help=help_msg["prefix"],
    )
    search_optional_args.add_argument(
        "--no-extend",
        dest="no_extend",
        action="store_true",
        help=help_msg["no_extend"],
        default=argparse.SUPPRESS,
    )
    extend_args.add_argument(
        "--extend-bin",
        default=defaults["extend_bin"],
        metavar="STR",
        type=lambda x: is_executable(parser, x, "--extend-bin"),
        dest="extend_bin",
        help=help_msg["extend_bin"],
    )
    extend_args.add_argument(
        "--extend-length",
        type=lambda x: int(
            check_values(x, minval=1, maxval=1000, parser=parser, var="--extend-length")
        ),
        metavar="INT",
        default=defaults["extend_length"],
        dest="extend_length",
        help=help_msg["extend_length"],
    )
    extend_args.add_argument(
        "--extend-k",
        type=lambda x: int(
            check_values(x, minval=9, maxval=121, parser=parser, var="--extend-k")
        ),
        metavar="INT",
        default=defaults["extend_k"],
        dest="extend_k",
        help=help_msg["extend_k"],
    )
    extend_args.add_argument(
        "--extend-memory",
        type=lambda x: check_suffix(x, parser=parser, var="--extend-memory"),
        default=defaults["extend_memory"],
        dest="extend_memory",
        metavar="INT",
        help=help_msg["extend_memory"],
    )
    search_optional_args.add_argument(
        "--no-derep",
        dest="no_derep",
        action="store_true",
        default=argparse.SUPPRESS,
        help=help_msg["no_derep"],
    )
    derep_args.add_argument(
        "--derep-bin",
        default=defaults["derep_bin"],
        metavar="STR",
        type=lambda x: is_in_derep_bins(parser, x, "--derep-bin"),
        dest="derep_bin",
        help=help_msg["derep_bin"],
    )
    # derep_args.add_argument(
    #     "--derep-min-length",
    #     type=lambda x: int(
    #         check_values(
    #             x, minval=1, maxval=100000, parser=parser, var="--derep-min-length"
    #         )
    #     ),
    #     default=defaults["derep_min_length"],
    #     dest="derep_min_length",
    #     help=help_msg["derep_min_length"],
    # )
    search_args.add_argument(
        "--mmseqs2-bin",
        default=defaults["mmseqs2_bin"],
        metavar="STR",
        type=lambda x: is_executable(parser, x, "--mmseqs2-bin"),
        dest="mmseqs_bin",
        help=help_msg["mmseqs_bin"],
    )
    search_args.add_argument(
        "--mmseq2-min-length",
        type=lambda x: int(
            check_values(
                x, minval=1, maxval=100000, parser=parser, var="--mmseqs2-min-length"
            )
        ),
        metavar="INT",
        default=defaults["mmseqs2_min_length"],
        dest="mmseqs_min_length",
        help=help_msg["mmseqs2_min_length"],
    )
    search_args.add_argument(
        "--mmseqs2-evalue",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1e6, parser=parser, var="--mmseqs2-evalue")
        ),
        metavar="FLOAT",
        default=defaults["mmseqs2_evalue"],
        dest="mmseqs_evalue",
        help=help_msg["evalue"],
    )
    search_args.add_argument(
        "--mmseq2-min-seqid",
        type=lambda x: float(
            check_values(
                x, minval=0, maxval=1, parser=parser, var="--mmseqs2-min-seqid"
            )
        ),
        metavar="FLOAT",
        default=defaults["mmseqs2_min_seqid"],
        dest="mmseqs_min_seqid",
        help=help_msg["mmseqs2_min_seqid"],
    )
    search_args.add_argument(
        "--mmseq2-cov",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1, parser=parser, var="--mmseqs2-cov")
        ),
        metavar="FLOAT",
        default=defaults["mmseqs2_cov"],
        dest="mmseqs_cov",
        help=help_msg["mmseqs2_cov"],
    )
    search_args.add_argument(
        "--mmseq2-cov-mode",
        type=lambda x: int(
            check_values(x, minval=0, maxval=5, parser=parser, var="--mmseqs2-cov-mode")
        ),
        metavar="INT",
        default=defaults["mmseqs2_cov_mode"],
        dest="mmseqs_cov_mode",
        help=help_msg["mmseqs2_cov_mode"],
    )
    search_optional_args.add_argument(
        "--ancient",
        dest="ancient",
        action="store_true",
        help=help_msg["ancient"],
        default=argparse.SUPPRESS,
    )
    search_optional_args.add_argument(
        "--keep",
        dest="no_keep",
        action="store_true",
        help=help_msg["keep"],
        default=argparse.SUPPRESS,
    )
    search_optional_args.add_argument(
        "--no-filter",
        dest="no_filter",
        action="store_true",
        help=help_msg["no_filter"],
        default=argparse.SUPPRESS,
    )
    filter_args.add_argument(
        "--x-filter-bin",
        default=defaults["xfilter_bin"],
        metavar="STR",
        type=lambda x: is_executable(parser, x, "--x-filter-bin"),
        dest="xfilter_bin",
        help=help_msg["xfilter_bin"],
    )
    filter_args.add_argument(
        "--n-iters",
        type=lambda x: int(
            check_values(x, minval=1, maxval=100000, parser=parser, var="--n-iters")
        ),
        default=defaults["iters"],
        metavar="INT",
        dest="iters",
        help=help_msg["iters"],
    )
    filter_args.add_argument(
        "--evalue",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1e6, parser=parser, var="--evalue")
        ),
        default=defaults["evalue"],
        metavar="FLOAT",
        dest="evalue",
        help=help_msg["evalue"],
    )
    filter_args.add_argument(
        "--scale",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1, parser=parser, var="--scale")
        ),
        default=defaults["scale"],
        metavar="FLOAT",
        dest="scale",
        help=help_msg["scale"],
    )
    filter_args.add_argument(
        "--bitscore",
        type=lambda x: int(
            check_values(x, minval=0, maxval=1e6, parser=parser, var="--bitscore")
        ),
        default=defaults["bitscore"],
        metavar="INT",
        dest="bitscore",
        help=help_msg["bitscore"],
    )
    filter_args.add_argument(
        "--filter",
        type=lambda x: str(check_filter_values(x, parser=parser, var="--filter")),
        default=defaults["filter"],
        metavar="STR",
        dest="filter",
        help=help_msg["filter"],
    )
    filter_args.add_argument(
        "--breadth",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1, parser=parser, var="--breadth")
        ),
        default=defaults["breadth"],
        metavar="FLOAT",
        dest="breadth",
        help=help_msg["breadth"],
    )
    filter_args.add_argument(
        "--breadth-expected-ratio",
        type=lambda x: float(
            check_values(
                x, minval=0, maxval=1, parser=parser, var="--breadth-expected-ratio"
            )
        ),
        default=defaults["breadth_expected_ratio"],
        metavar="FLOAT",
        dest="breadth_expected_ratio",
        help=help_msg["breadth_expected_ratio"],
    )
    filter_args.add_argument(
        "--depth",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1e6, parser=parser, var="--depth")
        ),
        default=defaults["depth"],
        metavar="FLOAT",
        dest="depth",
        help=help_msg["depth"],
    )
    filter_args.add_argument(
        "--depth-evenness",
        type=lambda x: float(
            check_values(x, minval=0, maxval=1e6, parser=parser, var="--depth-evenness")
        ),
        default=defaults["depth_evenness"],
        metavar="FLOAT",
        dest="depth_evenness",
        help=help_msg["depth_evenness"],
    )
    filter_args.add_argument(
        "--no-trim",
        dest="trim",
        action="store_false",
        help=help_msg["trim"],
        default=argparse.SUPPRESS,
    )
    filter_args.add_argument(
        "--no-aggregate",
        dest="agg",
        action="store_true",
        help=help_msg["agg"],
        default=argparse.SUPPRESS,
    )
    extract_args.add_argument(
        "--extract",
        dest="extract",
        action="store_true",
        help=help_msg["extract"],
        default=argparse.SUPPRESS,
    )
    extract_args.add_argument(
        "--extract-bin",
        default=defaults["extract_bin"],
        metavar="STR",
        type=lambda x: is_executable(parser, x, "--extract-bin"),
        dest="extract_bin",
        help=help_msg["extract_bin"],
    )
    args = parser.parse_args(None if sys.argv[1:] else ["-h"])

    if not hasattr(args, "no_derep"):
        args.no_derep = False
    if not hasattr(args, "no_extend"):
        args.no_extend = False
    if not hasattr(args, "no_filter"):
        args.no_filter = False
    if not hasattr(args, "trim"):
        args.trim = True
    if not hasattr(args, "ancient"):
        args.ancient = False
    if not hasattr(args, "no_keep"):
        args.no_keep = True
    if not hasattr(args, "recreate"):
        args.recreate = False
    if not hasattr(args, "local_db"):
        args.local_db = False
    if not hasattr(args, "agg"):
        args.agg = True
    if not hasattr(args, "extract"):
        args.extract = False

    if args.action is not None and len(sys.argv) == 2:
        args = parser.parse_args([args.action, "-h"])

    return args


@contextmanager
def suppress_stdout():
    """A context manager that redirects stdout and stderr to devnull"""
    with open(devnull, "w") as fnull:
        with redirect_stderr(fnull) as err, redirect_stdout(fnull) as out:
            yield (err, out)


def fast_flatten(input_list):
    return list(chain.from_iterable(input_list))


def concat_df(frames):
    COLUMN_NAMES = frames[0].columns
    df_dict = dict.fromkeys(COLUMN_NAMES, [])
    for col in COLUMN_NAMES:
        extracted = (frame[col] for frame in frames)
        # Flatten and save to df_dict
        df_dict[col] = fast_flatten(extracted)
    df = pd.DataFrame.from_dict(df_dict)[COLUMN_NAMES]
    return df


def is_fastq(filename):
    with get_open_func(filename)(filename, "rt") as handle:
        fastq = SeqIO.parse(handle, "fastq")
        return any(fastq)  # False when `fasta` is empty, i.e. wasn't a FASTA file


def create_output_files(prefix, input, db, fastq):

    if prefix is None:
        prefix = Path(input).resolve().stem.split(".")[0]
    if fastq:
        extension = "fq"
    else:
        extension = "fa"

    # create output files
    out_files = {
        "extended_reads": f"{prefix}.extended.fq.gz",
        "extend_reads_log": f"{prefix}.extend.log",
        "derep_reads": f"{prefix}.derep.fq.gz",
        "derep_reads_log": f"{prefix}.derep.log",
        "results": f"{prefix}.results.{db}.tsv",
        "results_gz": f"{prefix}.results.{db}.tsv.gz",
        "results_log": f"{prefix}.results.{db}.log",
        "results_filtered_prefix": f"{prefix}.results-filtered.{db}",
        "results_filtered_log": f"{prefix}.results-filtered.{db}.log",
        "results_filtered_mm": f"{prefix}.results-filtered.{db}_no-multimap.tsv.gz",
        "results_filtered_cov": f"{prefix}.results-filtered.{db}_cov-stats.tsv.gz",
        "results_filtered_group_agg": f"{prefix}.results-filtered.{db}_group-abundances-agg.tsv.gz",
        "results_filtered_group": f"{prefix}.results-filtered.{db}_group-abundances.tsv.gz",
        "reads_db_log": f"{prefix}.reads.{db}.log",
        "reads_db": f"{prefix}.reads.{db}.{extension}.gz",
    }
    return out_files



"""
This program is free software: you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""

import logging
from get_vf.createdb import createdb
from get_vf.utils import (
    get_arguments,
)
from get_vf.search import search_db

log = logging.getLogger("my_logger")


def main():

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(levelname)s ::: %(asctime)s ::: %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    args = get_arguments()
    logging.getLogger("my_logger").setLevel(
        logging.DEBUG if args.debug else logging.INFO
    )
    if args.action == "createdb":
        createdb(args)
    elif args.action == "search":
        search_db(args)


if __name__ == "__main__":
    main()


# import logging
# import pathlib
# import pandas as pd
# import os
# from get_vf.utils import (
#     get_arguments,
#     download,
#     create_folder_and_delete,
#     create_folder,
# )


# log = logging.getLogger("my_logger")


# def main():

#     logging.basicConfig(
#         level=logging.DEBUG, format="%(levelname)s ::: %(asctime)s ::: %(message)s"
#     )

#     args = get_arguments()

#     args = get_arguments()
#     logging.getLogger("my_logger").setLevel(
#         logging.DEBUG if args.debug else logging.INFO
#     )
#     if args.action == "split":
#         split_contigs(args)
#     elif args.action == "merge":
#         merge_contigs(args)

#     logging.getLogger("my_logger").setLevel(
#         logging.DEBUG if args.debug else logging.INFO
#     )
#     create_folder(args.outdir)
#     tmp_dir = f"{args.outdir}/tmp"
#     create_folder_and_delete(tmp_dir)

#     BASE_URL = f"http://files.metagenomics.eu//molecular_clock/DB/latest/"
#     # get list of marker genes
#     # check if metadata folder exists if not create it else delete it
#     metadata_dir = f"{args.outdir}/metadata"
#     metadata_file = pathlib.Path(metadata_dir, "bac120_marker_info.tsv")
#     # check if metadata file exists

#     if not os.path.exists(metadata_file):
#         create_folder(metadata_dir)
#         download(
#             f"{BASE_URL}/metadata/bac120_marker_info.tsv", dest_folder=metadata_dir
#         )
#     # get list of MSA genes
#     markers_df = pd.read_csv(metadata_file, sep="\t")
#     print(markers_df)


# if __name__ == "__main__":
#     main()



from setuptools import setup
import versioneer

requirements = [
    "pandas>=1.3.4",
    "scipy>=1.7.3",
    "tqdm>=4.62.3",
    "numpy>=1.21.4",
    "biopython>=1.79",
    "requests>=2.26.0",
]

setup(
    setup_requires=[
        # Setuptools 18.0 properly handles Cython extensions.
        "setuptools>=18.0",
        "Cython>=0.29.24",
    ],
    name="get-ancient-vf",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="A simple tool to map ancient reads against VFDB.",
    license="GNUv3",
    author="Antonio Fernandez-Guerra",
    author_email="antonio@metagenomics.eu",
    url="https://github.com/genomewalker/get-ancient-vf",
    packages=["get_vf"],
    entry_points={"console_scripts": ["getVF=get_vf.__main__:main"]},
    install_requires=requirements,
    keywords="get-ancient-vf",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
